@@ -1,4 +1,5 @@
 import logging
+import os
 import random
 import warnings
 from multiprocessing import cpu_count
@@ -136,12 +137,31 @@ class MultiLabelClassificationModel(ClassificationModel):
         else:
             self.device = "cpu"
 
-        if self.pos_weight:
-            self.model = model_class.from_pretrained(
-                model_name, config=self.config, pos_weight=torch.Tensor(self.pos_weight).to(self.device), **kwargs
-            )
+        if not self.args.quantized_model:
+            if self.pos_weight:
+                self.model = model_class.from_pretrained(
+                    model_name, config=self.config, pos_weight=torch.Tensor(self.pos_weight).to(self.device), **kwargs
+                )
+            else:
+                self.model = model_class.from_pretrained(model_name, config=self.config, **kwargs)
         else:
-            self.model = model_class.from_pretrained(model_name, config=self.config, **kwargs)
+            quantized_weights = torch.load(os.path.join(model_name, "pytorch_model.bin"))
+            if self.pos_weight:
+                self.model = model_class.from_pretrained(
+                    None,
+                    config=self.config,
+                    state_dict=quantized_weights,
+                    weight=torch.Tensor(self.pos_weight).to(self.device),
+                )
+            else:
+                self.model = model_class.from_pretrained(None, config=self.config, state_dict=quantized_weights)
+
+        if self.args.dynamic_quantize:
+            self.model = torch.quantization.quantize_dynamic(self.model, {torch.nn.Linear}, dtype=torch.qint8)
+        if self.args.quantized_model:
+            self.model.load_state_dict(quantized_weights)
+        if self.args.dynamic_quantize:
+            self.args.quantized_model = True
 
         self.results = {}
 
