@@ -12,11 +12,14 @@ import random
 import statistics
 import warnings
 from collections import defaultdict
+from dataclasses import asdict
 from itertools import chain
 from multiprocessing import cpu_count
-from dataclasses import asdict
 
 import numpy as np
+import pandas as pd
+import torch
+import torch.nn.functional as F
 from scipy.stats import mode, pearsonr
 from sklearn.metrics import (
     confusion_matrix,
@@ -25,18 +28,10 @@ from sklearn.metrics import (
     matthews_corrcoef,
     mean_squared_error,
 )
-from tqdm.auto import tqdm, trange
-
-import pandas as pd
-import torch
-import torch.nn.functional as F
-from simpletransformers.classification.classification_utils import InputExample, convert_examples_to_features
-from simpletransformers.config.global_args import global_args
-from simpletransformers.config.model_args import ConvAIArgs
-from simpletransformers.conv_ai.conv_ai_utils import get_dataset
 from tensorboardX import SummaryWriter
 from torch.utils.data import DataLoader, RandomSampler, SequentialSampler, TensorDataset
 from torch.utils.data.distributed import DistributedSampler
+from tqdm.auto import tqdm, trange
 from transformers import (
     WEIGHTS_NAME,
     AdamW,
@@ -49,6 +44,11 @@ from transformers import (
     get_linear_schedule_with_warmup,
 )
 
+from simpletransformers.classification.classification_utils import InputExample, convert_examples_to_features
+from simpletransformers.config.global_args import global_args
+from simpletransformers.config.model_args import ConvAIArgs
+from simpletransformers.conv_ai.conv_ai_utils import get_dataset
+
 try:
     import wandb
 
@@ -115,10 +115,24 @@ class ConvAIModel:
 
         config_class, model_class, tokenizer_class = MODEL_CLASSES[model_type]
         self.__dict__.update(kwargs)
+        self.config = config_class.from_pretrained(model_name, **self.args.config)
+
+        if not self.args.quantized_model:
+            self.model = model_class.from_pretrained(model_name, **kwargs)
+        else:
+            quantized_weights = torch.load(os.path.join(model_name, "pytorch_model.bin"))
+            self.model = model_class.from_pretrained(None, config=self.config, state_dict=quantized_weights)
 
-        self.model = model_class.from_pretrained(model_name, **kwargs)
         self.tokenizer = tokenizer_class.from_pretrained(model_name, **kwargs)
         self.add_special_tokens_(self.model, self.tokenizer)
+
+        if self.args.dynamic_quantize:
+            self.model = torch.quantization.quantize_dynamic(self.model, {torch.nn.Linear}, dtype=torch.qint8)
+        if self.args.quantized_model:
+            self.model.load_state_dict(quantized_weights)
+        if self.args.dynamic_quantize:
+            self.args.quantized_model = True
+
         self.results = {}
 
         if use_cuda:
@@ -183,9 +197,8 @@ class ConvAIModel:
         if os.path.exists(output_dir) and os.listdir(output_dir) and not self.args.overwrite_output_dir:
             raise ValueError(
                 "Output directory ({}) already exists and is not empty."
-                " Use --overwrite_output_dir to overcome.".format(output_dir)
+                " Set overwrite_output_dir: True to automatically overwrite.".format(output_dir)
             )
-
         self._move_model_to_device()
 
         train_dataloader, train_sampler = self.load_and_cache_examples(
@@ -208,7 +221,7 @@ class ConvAIModel:
             **kwargs,
         )
 
-        self._save_model(model=self.model)
+        self.save_model(model=self.model)
 
         if verbose:
             logger.info(" Training of {} model complete. Saved to {}.".format(self.args.model_type, output_dir))
@@ -292,14 +305,6 @@ class ConvAIModel:
             optimizer, num_warmup_steps=args.warmup_steps, num_training_steps=t_total
         )
 
-        if args.fp16:
-            try:
-                from apex import amp
-            except ImportError:
-                raise ImportError("Please install apex from https://www.github.com/nvidia/apex to use fp16 training.")
-
-            model, optimizer = amp.initialize(model, optimizer, opt_level=args.fp16_opt_level)
-
         if args.n_gpu > 1:
             model = torch.nn.DataParallel(model)
 
@@ -318,6 +323,11 @@ class ConvAIModel:
             wandb.init(project=args.wandb_project, config={**asdict(args)}, **args.wandb_kwargs)
             wandb.watch(self.model)
 
+        if args.fp16:
+            from torch.cuda import amp
+
+            scaler = amp.GradScaler()
+
         model.train()
         for _ in train_iterator:
             train_iterator.set_description(f"Epoch {epoch_number + 1} of {args.num_train_epochs}")
@@ -331,15 +341,27 @@ class ConvAIModel:
                 batch = tuple(t.to(device) for t in batch)
                 input_ids, mc_token_ids, lm_labels, mc_labels, token_type_ids = batch
 
-                (lm_loss), (mc_loss), *_ = model(
-                    input_ids,
-                    token_type_ids=token_type_ids,
-                    mc_token_ids=mc_token_ids,
-                    mc_labels=mc_labels,
-                    lm_labels=lm_labels,
-                )
-                # model outputs are always tuple in pytorch-transformers (see doc)
-                loss = lm_loss * args.lm_coef + mc_loss * args.mc_coef
+                if args.fp16:
+                    with amp.autocast():
+                        (lm_loss), (mc_loss), *_ = model(
+                            input_ids,
+                            token_type_ids=token_type_ids,
+                            mc_token_ids=mc_token_ids,
+                            mc_labels=mc_labels,
+                            lm_labels=lm_labels,
+                        )
+                        # model outputs are always tuple in pytorch-transformers (see doc)
+                        loss = lm_loss * args.lm_coef + mc_loss * args.mc_coef
+                else:
+                    (lm_loss), (mc_loss), *_ = model(
+                        input_ids,
+                        token_type_ids=token_type_ids,
+                        mc_token_ids=mc_token_ids,
+                        mc_labels=mc_labels,
+                        lm_labels=lm_labels,
+                    )
+                    # model outputs are always tuple in pytorch-transformers (see doc)
+                    loss = lm_loss * args.lm_coef + mc_loss * args.mc_coef
 
                 if args.n_gpu > 1:
                     loss = loss.mean()  # mean() to average on multi-gpu parallel training
@@ -353,25 +375,21 @@ class ConvAIModel:
                     loss = loss / args.gradient_accumulation_steps
 
                 if args.fp16:
-                    with amp.scale_loss(loss, optimizer) as scaled_loss:
-                        scaled_loss.backward()
-                    # torch.nn.utils.clip_grad_norm_(
-                    #     amp.master_params(optimizer), args.max_grad_norm
-                    # )
+                    scaler.scale(loss).backward()
                 else:
                     loss.backward()
-                    # torch.nn.utils.clip_grad_norm_(
-                    #     model.parameters(), args.max_grad_norm
-                    # )
 
                 tr_loss += loss.item()
                 if (step + 1) % args.gradient_accumulation_steps == 0:
                     if args.fp16:
-                        torch.nn.utils.clip_grad_norm_(amp.master_params(optimizer), args.max_grad_norm)
-                    else:
-                        torch.nn.utils.clip_grad_norm_(model.parameters(), args.max_grad_norm)
+                        scaler.unscale_(optimizer)
+                    torch.nn.utils.clip_grad_norm_(model.parameters(), args.max_grad_norm)
 
-                    optimizer.step()
+                    if args.fp16:
+                        scaler.step(optimizer)
+                        scaler.update()
+                    else:
+                        optimizer.step()
                     scheduler.step()  # Update learning rate schedule
                     model.zero_grad()
                     global_step += 1
@@ -394,7 +412,7 @@ class ConvAIModel:
                         # Save model checkpoint
                         output_dir_current = os.path.join(output_dir, "checkpoint-{}".format(global_step))
 
-                        self._save_model(output_dir_current, model=model)
+                        self.save_model(output_dir_current, model=model)
 
                     if args.evaluate_during_training and (
                         args.evaluate_during_training_steps > 0
@@ -413,7 +431,7 @@ class ConvAIModel:
                         output_dir_current = os.path.join(output_dir, "checkpoint-{}".format(global_step))
 
                         if args.save_eval_checkpoints:
-                            self._save_model(output_dir_current, model=model, results=results)
+                            self.save_model(output_dir_current, model=model, results=results)
 
                         training_progress_scores["global_step"].append(global_step)
                         training_progress_scores["train_loss"].append(current_loss)
@@ -429,11 +447,11 @@ class ConvAIModel:
 
                         if not best_eval_metric:
                             best_eval_metric = results[args.early_stopping_metric]
-                            self._save_model(args.best_model_dir, model=model, results=results)
+                            self.save_model(args.best_model_dir, model=model, results=results)
                         if best_eval_metric and args.early_stopping_metric_minimize:
                             if results[args.early_stopping_metric] - best_eval_metric < args.early_stopping_delta:
                                 best_eval_metric = results[args.early_stopping_metric]
-                                self._save_model(args.best_model_dir, model=model, results=results)
+                                self.save_model(args.best_model_dir, model=model, results=results)
                                 early_stopping_counter = 0
                             else:
                                 if args.use_early_stopping:
@@ -452,7 +470,7 @@ class ConvAIModel:
                         else:
                             if results[args.early_stopping_metric] - best_eval_metric > args.early_stopping_delta:
                                 best_eval_metric = results[args.early_stopping_metric]
-                                self._save_model(args.best_model_dir, model=model, results=results)
+                                self.save_model(args.best_model_dir, model=model, results=results)
                                 early_stopping_counter = 0
                             else:
                                 if args.use_early_stopping:
@@ -476,14 +494,14 @@ class ConvAIModel:
                 os.makedirs(output_dir_current, exist_ok=True)
 
             if args.save_model_every_epoch:
-                self._save_model(output_dir_current, model=model)
+                self.save_model(output_dir_current, model=model)
 
             if args.evaluate_during_training:
                 results, _, _ = self.eval_model(
                     eval_dataloader, verbose=verbose and args.evaluate_during_training_verbose, silent=True, **kwargs,
                 )
 
-                self._save_model(output_dir_current, results=results)
+                self.save_model(output_dir_current, results=results)
 
                 training_progress_scores["global_step"].append(global_step)
                 training_progress_scores["train_loss"].append(current_loss)
@@ -497,16 +515,16 @@ class ConvAIModel:
 
                 if not best_eval_metric:
                     best_eval_metric = results[args.early_stopping_metric]
-                    self._save_model(args.best_model_dir, model=model, results=results)
+                    self.save_model(args.best_model_dir, model=model, results=results)
                 if best_eval_metric and args.early_stopping_metric_minimize:
                     if results[args.early_stopping_metric] - best_eval_metric < args.early_stopping_delta:
                         best_eval_metric = results[args.early_stopping_metric]
-                        self._save_model(args.best_model_dir, model=model, results=results)
+                        self.save_model(args.best_model_dir, model=model, results=results)
                         early_stopping_counter = 0
                 else:
                     if results[args.early_stopping_metric] - best_eval_metric > args.early_stopping_delta:
                         best_eval_metric = results[args.early_stopping_metric]
-                        self._save_model(args.best_model_dir, model=model, results=results)
+                        self.save_model(args.best_model_dir, model=model, results=results)
                         early_stopping_counter = 0
 
         return global_step, tr_loss / global_step
@@ -615,8 +633,7 @@ class ConvAIModel:
         if not no_cache:
             no_cache = args.no_cache
 
-        if not no_cache:
-            os.makedirs(self.args.cache_dir, exist_ok=True)
+        os.makedirs(self.args.cache_dir, exist_ok=True)
 
         dataset_path = dataset_path if dataset_path else ""
 
@@ -765,7 +782,7 @@ class ConvAIModel:
                             The history will be tokenized and encoded.
 
         Returns:
-            out_text: the response generated by the model based on the personality, history and message.
+            out_text: The response generated by the model based on the personality, history and message.
             history: The updated history of the conversation. If encode_history is True, this will be in text form.
                         If not, it will be in encoded form.
         """
@@ -834,7 +851,7 @@ class ConvAIModel:
 
         return training_progress_scores
 
-    def _save_model(self, output_dir=None, model=None, results=None):
+    def save_model(self, output_dir=None, model=None, results=None):
         if not output_dir:
             output_dir = self.args.output_dir
 
@@ -843,7 +860,7 @@ class ConvAIModel:
             model_to_save = model.module if hasattr(model, "module") else model
             model_to_save.save_pretrained(output_dir)
             self.tokenizer.save_pretrained(output_dir)
-            self._save_model_args(output_dir)
+            self.save_model_args(output_dir)
 
         if results:
             output_eval_file = os.path.join(output_dir, "eval_results.txt")
@@ -954,7 +971,7 @@ class ConvAIModel:
 
         return current_output
 
-    def _save_model_args(self, output_dir):
+    def save_model_args(self, output_dir):
         os.makedirs(output_dir, exist_ok=True)
         self.args.save(output_dir)
 
